/*
 * Copyright 2004-2010 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.doma.jdbc.tx;

import java.sql.Connection;

/**
 * トランザクション分離レベルを示します。
 * 
 * @author taedium
 * @since 1.1.0
 */
public enum TransactionIsolationLevel {

    /** @see Connection#TRANSACTION_READ_UNCOMMITTED　 */
    READ_UNCOMMITTED(Connection.TRANSACTION_READ_UNCOMMITTED),

    /** @see Connection#TRANSACTION_READ_COMMITTED */
    READ_COMMITTED(Connection.TRANSACTION_READ_COMMITTED),

    /** @see Connection#TRANSACTION_REPEATABLE_READ */
    REPEATABLE_READ(Connection.TRANSACTION_REPEATABLE_READ),

    /** @see Connection#TRANSACTION_SERIALIZABLE */
    SERIALIZABLE(Connection.TRANSACTION_SERIALIZABLE);

    private int level;

    private TransactionIsolationLevel(int level) {
        this.level = level;
    }

    /**
     * トランザクション分離レベルをあらわす定数を返します。
     * 
     * @return トランザクション分離レベルをあらわす定数
     * @see Connection#TRANSACTION_READ_UNCOMMITTED
     * @see Connection#TRANSACTION_READ_COMMITTED
     * @see Connection#TRANSACTION_REPEATABLE_READ
     * @see Connection#TRANSACTION_SERIALIZABLE
     */
    public int getLevel() {
        return level;
    }
}
